@@ -26,6 +26,8 @@
 #define VOLUME_NORM ((uint32_t) 0x10000U)
 #define VOLUME_MAX ((uint32_t) UINT32_MAX/2)
 
+#define PA_CHANNELS_MAX	(32u)
+
 static inline uint32_t volume_from_linear(float vol)
 {
 	uint32_t v;
@@ -86,14 +88,6 @@ static inline const struct str_map *str_map_find(const struct str_map *map, cons
 	return NULL;
 }
 
-struct descriptor {
-	uint32_t length;
-	uint32_t channel;
-	uint32_t offset_hi;
-	uint32_t offset_lo;
-	uint32_t flags;
-};
-
 enum {
 	TAG_INVALID = 0,
 	TAG_STRING = 't',
@@ -326,7 +320,7 @@ static int read_format_info(struct message *m, struct format_info *info)
 static int message_get(struct message *m, ...)
 {
 	va_list va;
-	int res;
+	int res = 0;
 
 	va_start(va, m);
 
@@ -337,107 +331,114 @@ static int message_get(struct message *m, ...)
 			break;
 
 		if ((res = read_u8(m, &dtag)) < 0)
-			return res;
+			goto done;
 
 		switch (dtag) {
 		case TAG_STRING:
 			if (tag != TAG_STRING)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_string(m, va_arg(va, char**))) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_STRING_NULL:
 			if (tag != TAG_STRING)
-				return -EINVAL;
+				goto invalid;
 			*va_arg(va, char**) = NULL;
 			break;
 		case TAG_U8:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_u8(m, va_arg(va, uint8_t*))) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_U32:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_u32(m, va_arg(va, uint32_t*))) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_S64:
 		case TAG_U64:
 		case TAG_USEC:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_u64(m, va_arg(va, uint64_t*))) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_SAMPLE_SPEC:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_sample_spec(m, va_arg(va, struct sample_spec*))) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_ARBITRARY:
 		{
 			const void **val = va_arg(va, const void**);
 			size_t *len = va_arg(va, size_t*);
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_arbitrary(m, val, len)) < 0)
-				return res;
+				goto done;
 			break;
 		}
 		case TAG_BOOLEAN_TRUE:
 			if (tag != TAG_BOOLEAN)
-				return -EINVAL;
+				goto invalid;
 			*va_arg(va, bool*) = true;
 			break;
 		case TAG_BOOLEAN_FALSE:
 			if (tag != TAG_BOOLEAN)
-				return -EINVAL;
+				goto invalid;
 			*va_arg(va, bool*) = false;
 			break;
 		case TAG_TIMEVAL:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_timeval(m, va_arg(va, struct timeval*))) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_CHANNEL_MAP:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_channel_map(m, va_arg(va, struct channel_map*))) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_CVOLUME:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_cvolume(m, va_arg(va, struct volume*))) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_PROPLIST:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_props(m, va_arg(va, struct pw_properties*), true)) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_VOLUME:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_volume(m, va_arg(va, float*))) < 0)
-				return res;
+				goto done;
 			break;
 		case TAG_FORMAT_INFO:
 			if (dtag != tag)
-				return -EINVAL;
+				goto invalid;
 			if ((res = read_format_info(m, va_arg(va, struct format_info*))) < 0)
-				return res;
+				goto done;
 			break;
 		}
 	}
+	res = 0;
+	goto done;
+
+invalid:
+	res = -EINVAL;
+
+done:
 	va_end(va);
 
-	return 0;
+	return res;
 }
 
 static int ensure_size(struct message *m, uint32_t size)
@@ -505,9 +506,10 @@ static void write_64(struct message *m, uint8_t tag, uint64_t val)
 
 static void write_sample_spec(struct message *m, struct sample_spec *ss)
 {
+	uint32_t channels = SPA_MIN(ss->channels, PA_CHANNELS_MAX);
 	write_8(m, TAG_SAMPLE_SPEC);
 	write_8(m, format_id2pa(ss->format));
-	write_8(m, ss->channels);
+	write_8(m, channels);
 	write_32(m, ss->rate);
 }
 
@@ -535,10 +537,10 @@ static void write_timeval(struct message *m, struct timeval *tv)
 static void write_channel_map(struct message *m, struct channel_map *map)
 {
 	uint8_t i;
-	uint32_t aux = 0;
+	uint32_t aux = 0, channels = SPA_MIN(map->channels, PA_CHANNELS_MAX);
 	write_8(m, TAG_CHANNEL_MAP);
-	write_8(m, map->channels);
-	for (i = 0; i < map->channels; i ++)
+	write_8(m, channels);
+	for (i = 0; i < channels; i ++)
 		write_8(m, channel_id2pa(map->map[i], &aux));
 }
 
@@ -551,9 +553,10 @@ static void write_volume(struct message *m, float vol)
 static void write_cvolume(struct message *m, struct volume *vol)
 {
 	uint8_t i;
+	uint32_t channels = SPA_MIN(vol->channels, PA_CHANNELS_MAX);
 	write_8(m, TAG_CVOLUME);
-	write_8(m, vol->channels);
-	for (i = 0; i < vol->channels; i ++)
+	write_8(m, channels);
+	for (i = 0; i < channels; i ++)
 		write_32(m, volume_from_linear(vol->values[i]));
 }
 
@@ -828,12 +831,15 @@ static int message_dump(enum spa_log_level level, struct message *m)
 		{
 			struct pw_properties *props = pw_properties_new(NULL, NULL);
 			const struct spa_dict_item *it;
-			if ((res = read_props(m, props, false)) < 0)
-				return res;
-			pw_log(level, "%u: props: n_items:%u", o, props->dict.n_items);
-			spa_dict_for_each(it, &props->dict)
-				pw_log(level, "     '%s': '%s'", it->key, it->value);
+			res = read_props(m, props, false);
+			if (res >= 0) {
+				pw_log(level, "%u: props: n_items:%u", o, props->dict.n_items);
+				spa_dict_for_each(it, &props->dict)
+					pw_log(level, "     '%s': '%s'", it->key, it->value);
+			}
 			pw_properties_free(props);
+			if (res < 0)
+				return res;
 			break;
 		}
 		case TAG_VOLUME:
